@@ -30,18 +30,54 @@ except ImportError:
     except ImportError:
         print "Error: Neither sqlite3 nor pysqlite2 found"
         raise
+
 import os, time
 import datetime
-import hamster
-import hamster.storage
-from hamster import stuff
+import storage
+import stuff
+from shutil import copy as copyfile
 import datetime as dt
-import copy
-        
-class Storage(hamster.storage.Storage):
-    # we are saving data under $HOME/.gnome2/hamster-applet/hamster.db
+import gettext
+
+DB_FILE = 'hamster.db'
+
+class Storage(storage.Storage):
     con = None # Connection will be created on demand
 
+    def __setup(self):
+        """
+        Delayed setup so we don't do everything at the same time
+        """
+        if self.__setup.im_func.complete:
+            return
+
+        from configuration import runtime
+
+        db_file = runtime.database_file
+        db_path, _ = os.path.split(os.path.realpath(db_file))
+
+        if not os.path.exists(db_path):
+            try:
+                os.makedirs(db_path, 0744)
+            except Exception, msg:
+                print 'Error:could not create user dir (%s): %s' % (db_path, msg)
+
+        data_dir = runtime.data_dir
+
+        #check if db is here
+        if not os.path.exists(db_file):
+            print "Database not found in %s - installing default from %s!" % (db_file, data_dir)
+            copyfile(os.path.join(data_dir, DB_FILE), db_file)
+
+            #change also permissions - sometimes they are 444
+            try:
+                os.chmod(db_file, 0664)
+            except Exception, msg:
+                print 'Error:could not change mode on %s!' % (db_file)
+        self.__setup.im_func.complete = True
+        self.run_fixtures()
+    __setup.complete = False
+
     def __get_category_list(self):
         return self.fetchall("SELECT * FROM categories ORDER BY category_order")
 
@@ -226,22 +262,51 @@ class Storage(hamster.storage.Storage):
             """
             self.execute(query, (end_time, fact['id']))
 
-    def __solve_overlaps(self, start_time, end_time):
-        """finds facts that happen in given interval and shifts them to
-        make room for new fact"""
-        
-        # in case of missing end_time, treat is as ongoing task, but only
-        # if that's not too far in past
-        if not end_time and (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
-            end_time = dt.datetime.now()
-            
-        
-        # activities that we are overlapping.
-        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+    def __squeeze_in(self, start_time):
+        # tries to put task in the given date
+        # if there are conflicts, we will only truncate the ongoing task
+        # and replace it's end part with our activity
+
+        # we are checking if our start time is in the middle of anything
+        # or maybe there is something after us - so we know to adjust end time
         query = """
                    SELECT a.*, b.name
                      FROM facts a
                 LEFT JOIN activities b on b.id = a.activity_id
+                    WHERE ((start_time < ? and end_time > ?) or start_time > ?)
+                 ORDER BY start_time
+                    LIMIT 1
+                """
+        fact = self.fetchone(query, (start_time, start_time, start_time))
+
+        end_time = None        
+
+        if fact:
+            if fact["start_time"] < start_time and fact["end_time"]:
+                #we are in middle of a task
+                self.execute("UPDATE facts SET end_time=? WHERE id=?",
+                             (start_time, fact["id"]))
+                end_time = fact["end_time"]
+            else: #otherwise we have found a task that is after us
+                end_time = fact["start_time"]
+
+        return end_time
+        
+    def __solve_overlaps(self, start_time, end_time):
+        """finds facts that happen in given interval and shifts them to
+        make room for new fact"""
+
+        # this function is destructive - can't go with a wildcard
+        if not end_time or not start_time: #this function is destructive
+            return
+        
+        # activities that we are overlapping.
+        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+        query = """
+                   SELECT a.*, b.name, c.name as category
+                     FROM facts a
+                LEFT JOIN activities b on b.id = a.activity_id
+                LEFT JOIN categories c on b.category_id = c.id
                     WHERE ((start_time < ? and end_time > ?)
                            OR (start_time < ? and end_time > ?))
                            
@@ -261,7 +326,12 @@ class Storage(hamster.storage.Storage):
                 self.execute("""UPDATE facts
                                    SET end_time = ?
                                  WHERE id = ?""", (start_time, fact["id"]))
-                self.__add_fact(fact["name"], end_time, fact["end_time"])
+                fact_name = fact["name"]
+                self.__add_fact(fact["name"],
+                                end_time,
+                                fact["end_time"],
+                                fact["category"],
+                                fact["description"])
 
             #eliminate
             elif fact["end_time"] and \
@@ -283,9 +353,20 @@ class Storage(hamster.storage.Storage):
                              (start_time, fact["id"]))
 
 
-    def __add_fact(self, activity_name, start_time = None, end_time = None):
+    def __add_fact(self, activity_name, start_time = None,
+                     end_time = None, category_name = None, description = None):
         activity = stuff.parse_activity_input(activity_name)
+        
+        if category_name:
+            activity.category_name = category_name
+        if description:
+            activity.description = description
+        
         start_time = activity.start_time or start_time or datetime.datetime.now()
+        
+        if start_time > datetime.datetime.now():
+            return None #no facts in future, please
+        
         start_time = start_time.replace(microsecond = 0)
         end_time = activity.end_time or end_time
         if end_time:
@@ -317,13 +398,14 @@ class Storage(hamster.storage.Storage):
                    and last_activity['activity_id'] == activity_id:
                     return last_activity
                 
+                #if duration is less than a minute - it must have been a mistake
                 if not activity.description \
                    and not last_activity["description"] \
                    and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
                     self.__remove_fact(last_activity['id'])
                     start_time = last_activity['start_time']
                 else:
-                    #stop 
+                    #otherwise stop 
                     update = """
                                UPDATE facts
                                   SET end_time = ?
@@ -332,8 +414,11 @@ class Storage(hamster.storage.Storage):
                     self.execute(update, (start_time, last_activity["id"]))
 
 
-        #done with the current activity, now we can solve overlaps
-        self.__solve_overlaps(start_time, end_time)
+        # done with the current activity, now we can solve overlaps
+        if not end_time:
+            end_time = self.__squeeze_in(start_time)
+        else:
+            self.__solve_overlaps(start_time, end_time)
 
 
         # finally add the new entry
@@ -362,8 +447,10 @@ class Storage(hamster.storage.Storage):
                     WHERE (a.end_time >= ? OR a.end_time IS NULL) AND a.start_time <= ?
         """
         
-        if category_id:
+        if category_id and isinstance(category_id, int):
             query += " and b.category_id = %d" % category_id
+        elif category_id and isinstance(category_id, list):
+            query += " and b.category_id IN (%s)" % (",".join([str(id) for id in category_id]))
 
         query += " ORDER BY a.start_time"
         end_date = end_date or date
@@ -448,21 +535,6 @@ class Storage(hamster.storage.Storage):
         """
         return self.fetchall(query, (_("Unsorted"), ))
 
-    def __get_interval_activity_ids(self, date, end_date = None):
-        """returns activities used in the specified interval"""
-        query = """
-                   SELECT a.name, coalesce(b.name, ?) as category_name
-                     FROM activities a
-                LEFT JOIN categories b on b.id = a.category_id
-                    WHERE a.id in (SELECT activity_id from facts
-                                    WHERE (date(start_time) >= ? and date(start_time) <= ?)
-                                       OR (date(end_time) >= ? and date(end_time) <= ?))
-        """
-        end_date = end_date or date        
-
-        return self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
-        
-    
     def __remove_fact(self, fact_id):
         query = """
                    DELETE FROM facts
@@ -587,8 +659,10 @@ class Storage(hamster.storage.Storage):
 
     """ Here be dragons (lame connection/cursor wrappers) """
     def get_connection(self):
+        from configuration import runtime
         if self.con is None:
-            self.con = sqlite.connect(hamster.HAMSTER_DB, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
+            db_file = runtime.database_file
+            self.con = sqlite.connect(db_file, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
         return self.con
@@ -596,10 +670,13 @@ class Storage(hamster.storage.Storage):
     connection = property(get_connection, None)
 
     def fetchall(self, query, params = None):
+        from configuration import runtime
+        self.__setup()
+
         con = self.connection
         cur = con.cursor()
 
-        if hamster.trace_sql:
+        if runtime.trace_sql:
             print query, params
 
         if params:
@@ -620,8 +697,13 @@ class Storage(hamster.storage.Storage):
             return None
 
     def execute(self, statement, params = ()):
-        """execute sql statement. optionally you can give multiple statements
-        to save on cursor creation and closure"""
+        """
+        execute sql statement. optionally you can give multiple statements
+        to save on cursor creation and closure
+        """
+        from configuration import runtime
+        self.__setup()
+
         con = self.connection
         cur = con.cursor()
         
@@ -631,7 +713,7 @@ class Storage(hamster.storage.Storage):
             
         if isinstance(statement, list):
             for i in range(len(statement)):
-                if hamster.trace_sql:
+                if runtime.trace_sql:
                     print statement[i], params[i]
          
                 res = cur.execute(statement[i], params[i])
@@ -840,7 +922,4 @@ class Storage(hamster.storage.Storage):
             nonwork_cat_id = self.__add_category(nonwork_category["name"])
             for entry in nonwork_category["entries"]:
                 self.__add_activity(entry, nonwork_cat_id)
-        
-        
-        
 
